/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.amap.lidar.amapvox.visualization.controllers.loadables;

/**
 * Abstract class used to load, bind and unload objects.
 * @author Tristan Muller (tristan.muller@cirad.fr)
 */
public abstract class Loadable {
        
    /**
     * If this has been loaded or not.
     */
    private boolean loaded;
    
    /**
     * Create a loadable object that is not loaded by default.
     */
    public Loadable() {
        this.loaded = false;
    }

    /**
     * Call-it to load this object (only if that has not been already loaded).
     */
    public final void load() {
        if (!loaded) {
            loaded = true;
            onLoad();
        }
    }
    
    /**
     * Implements-it to specify what appends when this object is loaded.
     */
    protected abstract void onLoad();
    
    /**
     * Call-it to bind (update) this object.
     */
    public final void bind() {
        if (!loaded) {
            load();
        } else {
            onBind();
        }
    }
    
    /**
     * Implements-it to specify what appends when this object is binded (updated).
     */
    protected abstract void onBind();
    
    /**
     * Call-it to unload this object (work only is that has already been loaded).
     */
    public final void unload() {
        if (loaded) {
            loaded = false;
            onUnload();
        }
    }
    
    /**
     * Implements-it to specify what appends when this object is unloaded.
     */
    protected abstract void onUnload();
    
}
