/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.amap.lidar.amapvox.visualization.controllers.loadables.meshes.extended;

import fr.amap.lidar.amapvox.visualization.controllers.loadables.meshes.base.Primitive;
import fr.amap.lidar.amapvox.visualization.controllers.loadables.meshes.base.Vertex;
import fr.amap.lidar.amapvox.visualization.controllers.loadables.meshes.faces.LineFace;
import fr.amap.lidar.amapvox.visualization.controllers.loadables.meshes.faces.PointFace;
import fr.amap.lidar.amapvox.visualization.controllers.loadables.meshes.faces.QuadFace;
import fr.amap.lidar.amapvox.visualization.controllers.loadables.meshes.faces.TriangleFace;

/**
 * Cube primitive class.
 * @author Tristan Muller (tristan.muller@cirad.fr)
 */
public class CubePrimitive extends Primitive {
    
    /**
     * Used to make a concurrent singleton.
     */
    private static class SingletonHolder {
        private static final CubePrimitive INSTANCE = new CubePrimitive();
    }

    /**
     * Get the singleton instance of this object.
     * @return 
     */
    public static final CubePrimitive getINSTANCE() {
        return SingletonHolder.INSTANCE;
    }
    
    @Override
    protected void generateVertices() {
        addVertex(new Vertex(1.0f, 1.0f, 1.0f));
        addVertex(new Vertex(1.0f, -1.0f, 1.0f));
        addVertex(new Vertex(1.0f, 1.0f, -1.0f));
        addVertex(new Vertex(-1.0f, 1.0f, 1.0f));
        addVertex(new Vertex(1.0f, -1.0f, -1.0f));
        addVertex(new Vertex(-1.0f, -1.0f, 1.0f));
        addVertex(new Vertex(-1.0f, 1.0f, -1.0f));
        addVertex(new Vertex(-1.0f, -1.0f, -1.0f));
    }

    @Override
    protected void generatePointFaces() {
        addPointFace(new PointFace(0));
        addPointFace(new PointFace(1));
        addPointFace(new PointFace(2));
        addPointFace(new PointFace(3));
        addPointFace(new PointFace(4));
        addPointFace(new PointFace(5));
        addPointFace(new PointFace(6));
        addPointFace(new PointFace(7));
    }

    @Override
    protected void generateLineFaces() {
        addLineFace(new LineFace(0, 1));
        addLineFace(new LineFace(0, 2));
        addLineFace(new LineFace(0, 3));
        addLineFace(new LineFace(1, 4));
        addLineFace(new LineFace(1, 5));
        addLineFace(new LineFace(2, 4));
        addLineFace(new LineFace(2, 6));
        addLineFace(new LineFace(3, 5));
        addLineFace(new LineFace(3, 6));
        addLineFace(new LineFace(7, 6));
        addLineFace(new LineFace(7, 4));
        addLineFace(new LineFace(7, 5));
    }

    @Override
    protected void generateTriangleFaces() {
        addTriangleFace(new TriangleFace(0, 1, 2));
        addTriangleFace(new TriangleFace(2, 1, 4));
        addTriangleFace(new TriangleFace(1, 5, 3));
        addTriangleFace(new TriangleFace(1, 3, 0));
        addTriangleFace(new TriangleFace(3, 7, 6));
        addTriangleFace(new TriangleFace(3, 5, 7));
        addTriangleFace(new TriangleFace(7, 4, 2));
        addTriangleFace(new TriangleFace(7, 2, 6));
        addTriangleFace(new TriangleFace(1, 4, 7));
        addTriangleFace(new TriangleFace(1, 7, 5));
        addTriangleFace(new TriangleFace(0, 6, 3));
        addTriangleFace(new TriangleFace(0, 6, 2));
    }

    @Override
    protected void generateQuadFaces() {
        addQuadFace(new QuadFace(0, 1, 4, 2));
        addQuadFace(new QuadFace(0, 3, 5, 1));
        addQuadFace(new QuadFace(1, 5, 7, 4));
        addQuadFace(new QuadFace(5, 3, 6, 7));
        addQuadFace(new QuadFace(6, 3, 0, 2));
        addQuadFace(new QuadFace(4, 7, 6, 2));
    }
    
}
